@@ -187,6 +187,7 @@ impl Rate {
         self.end_of_app_limited != 0
     }
 
+    #[cfg(test)]
     pub fn delivered(&self) -> usize {
         self.delivered
     }
@@ -195,21 +196,10 @@ impl Rate {
         self.rate_sample.bandwidth
     }
 
-    pub fn sample_rtt(&self) -> Duration {
-        self.rate_sample.rtt
-    }
-
+    #[cfg(test)]
     pub fn sample_is_app_limited(&self) -> bool {
         self.rate_sample.is_app_limited
     }
-
-    pub fn sample_delivered(&self) -> usize {
-        self.rate_sample.delivered
-    }
-
-    pub fn sample_prior_delivered(&self) -> usize {
-        self.rate_sample.prior_delivered
-    }
 }
 
 #[derive(Debug)]
@@ -259,7 +249,7 @@ mod tests {
     use crate::recovery::congestion::recovery::LegacyRecovery;
     use crate::recovery::HandshakeStatus;
     use crate::recovery::RecoveryOps;
-    use crate::testing;
+    use crate::test_utils;
     use crate::Config;
     use crate::OnAckReceivedOutcome;
     use std::ops::Range;
@@ -397,7 +387,7 @@ mod tests {
 
         // Send 2 packets.
         for pn in 0..2 {
-            let pkt = testing::helper_packet_sent(pn, now, mss);
+            let pkt = test_utils::helper_packet_sent(pn, now, mss);
 
             r.on_packet_sent(
                 pkt,
@@ -451,7 +441,7 @@ mod tests {
 
         // Send 10 packets to fill cwnd.
         for pn in 0..5 {
-            let pkt = testing::helper_packet_sent(pn, now, mss);
+            let pkt = test_utils::helper_packet_sent(pn, now, mss);
             r.on_packet_sent(
                 pkt,
                 packet::Epoch::Application,
@@ -466,7 +456,7 @@ mod tests {
         assert!(!r.congestion.delivery_rate.sample_is_app_limited());
 
         for pn in 5..10 {
-            let pkt = testing::helper_packet_sent(pn, now, mss);
+            let pkt = test_utils::helper_packet_sent(pn, now, mss);
             r.on_packet_sent(
                 pkt,
                 packet::Epoch::Application,
@@ -486,7 +476,7 @@ mod tests {
         rtt: Duration, mss: usize,
     ) {
         for pn in range.clone() {
-            let pkt = testing::helper_packet_sent(pn, now, mss);
+            let pkt = test_utils::helper_packet_sent(pn, now, mss);
             recovery.on_packet_sent(
                 pkt,
                 packet::Epoch::Application,
@@ -502,15 +492,17 @@ mod tests {
         let mut acked = ranges::RangeSet::default();
         acked.insert(range);
 
-        let ack_outcome = recovery.on_ack_received(
-            &acked,
-            25,
-            packet::Epoch::Application,
-            HandshakeStatus::default(),
-            now + rtt,
-            None,
-            "",
-        ).unwrap();
+        let ack_outcome = recovery
+            .on_ack_received(
+                &acked,
+                25,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now + rtt,
+                None,
+                "",
+            )
+            .unwrap();
 
         assert_eq!(ack_outcome, OnAckReceivedOutcome {
             lost_packets: 0,
